@@ -3,14 +3,28 @@
 
 import unittest
 from datetime import datetime, timedelta
+from functools import partial
 from time import sleep
 import pytz
-from croniter import croniter, CroniterBadDateError, CroniterBadCronError, CroniterNotAlphaError
+from croniter import croniter, croniter_range, CroniterBadDateError, CroniterBadCronError, CroniterNotAlphaError
 from croniter.tests import base
+from tzlocal import get_localzone
+import dateutil.tz
 
 
 class CroniterTest(base.TestCase):
 
+    def testSecondSec(self):
+        base = datetime(2012, 4, 6, 13, 26, 10)
+        itr = croniter('* * * * * 15,25', base)
+        n = itr.get_next(datetime)
+        self.assertEqual(15, n.second)
+        n = itr.get_next(datetime)
+        self.assertEqual(25, n.second)
+        n = itr.get_next(datetime)
+        self.assertEqual(15, n.second)
+        self.assertEqual(27, n.minute)
+
     def testSecond(self):
         base = datetime(2012, 4, 6, 13, 26, 10)
         itr = croniter('*/1 * * * * *', base)
@@ -285,6 +299,32 @@ class CroniterTest(base.TestCase):
         self.assertEqual(22, next.hour)
         self.assertEqual(30, next.minute)
 
+    def testOptimizeCronExpressions(self):
+        """ Non-optimal cron expressions that can be simplified."""
+        wildcard = ['*']
+        m, h, d, mon, dow, s = range(6)
+        # Test each field individually 
+        self.assertEqual(croniter('0-59 0 0 0 0').expanded[m], wildcard)
+        self.assertEqual(croniter('0 0-23 0 0 0').expanded[h], wildcard)
+        self.assertEqual(croniter('0 0 0-31 0 0').expanded[d], wildcard)
+        self.assertEqual(croniter('0 0 0 1-12 0').expanded[mon], wildcard)
+        self.assertEqual(croniter('0 0 0 0 0-6').expanded[dow], wildcard)
+        self.assertEqual(croniter('0 0 0 0 1-7').expanded[dow], wildcard)
+        self.assertEqual(croniter('0 0 0 0 0 0-59').expanded[s], wildcard)
+        # Real life examples
+        self.assertEqual(croniter('30 1-12,0,10-23 15-21 * fri').expanded[h], wildcard)
+        self.assertEqual(croniter('30 1-23,0 15-21 * fri').expanded[h], wildcard)
+
+    def testBlockDupRanges(self):
+        """ Ensure that duplicate/overlapping ranges are squashed """
+        m, h, d, mon, dow, s = range(6)
+        self.assertEqual(croniter('* 5,5,1-6 * * *').expanded[h], [1,2,3,4,5,6])
+        self.assertEqual(croniter('* * * * 2-3,4-5,3,3,3').expanded[dow], [2,3,4,5])
+        self.assertEqual(croniter('* * * * * 1,5,*/20,20,15').expanded[s], [0, 1, 5, 15, 20, 40])
+        self.assertEqual(croniter('* 4,1-4,5,4 * * *').expanded[h], [1, 2, 3, 4, 5])
+        # Real life example
+        self.assertEqual(croniter('59 23 * 1 wed,fri,mon-thu,tue,tue').expanded[dow], [1,2,3,4,5])
+
     def testPrevMinute(self):
         base = datetime(2010, 8, 25, 15, 56)
         itr = croniter('*/1 * * * *', base)
@@ -637,6 +677,13 @@ class CroniterTest(base.TestCase):
         n2 = itr2.get_next(datetime)
         self.assertEqual(n2.tzinfo.zone, 'Asia/Tokyo')
 
+    def testTimezoneDateutil(self):
+        tokyo = dateutil.tz.gettz('Asia/Tokyo')
+        base = datetime(2013, 3, 4, 12, 15, tzinfo=tokyo)
+        itr = croniter('* * * * *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.tzinfo.tzname(n1), 'JST')
+
     def testInitNoStartTime(self):
         itr = croniter('* * * * *')
         sleep(.01)
@@ -775,6 +822,27 @@ class CroniterTest(base.TestCase):
         lret1 = ['{0}'.format(d[1]) for d in local_dates]
         self.assertEqual(sret1, lret1)
 
+    def test_std_dst3(self):
+        """
+        DST tests
+
+        This fixes https://github.com/taichino/croniter/issues/90
+
+        Adelaide, Australia: 15/04/2020 00:00 -> 15/03/2020
+
+        """
+
+        tz = pytz.timezone('Australia/Adelaide')
+
+        schedule = croniter('0 0 24 * *', tz.localize(datetime(2020, 4,15)))
+        val1 = schedule.get_prev(datetime)
+        dt1 = tz.localize(datetime(2020, 3, 24))
+        self.assertEqual(val1, dt1)
+
+        val2 = schedule.get_next(datetime)
+        dt2 = tz.localize(datetime(2020, 4, 24))
+        self.assertEqual(val2, dt2)
+
     def test_error_alpha_cron(self):
         self.assertRaises(CroniterNotAlphaError, croniter.expand,
                           '* * * janu-jun *')
@@ -816,21 +884,38 @@ class CroniterTest(base.TestCase):
         """
         https://github.com/taichino/croniter/issues/107
         """
-        #
+
+        _croniter = partial(croniter, "0 10 * * *", ret_type=datetime)
+
         dt = datetime(2018, 1, 2, 10, 0, 0, 500)
-        c = croniter("0 10 * * * ", start_time=dt)
-        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
-        self.assertEqual(ts, '2018-01-02 10:00:00')
-        #
+        self.assertEqual(
+            _croniter(start_time=dt).get_prev(),
+            datetime(2018, 1, 2, 10, 0),
+        )
+        self.assertEqual(
+            _croniter(start_time=dt).get_next(),
+            datetime(2018, 1, 3, 10, 0),
+        )
+
         dt = datetime(2018, 1, 2, 10, 0, 1, 0)
-        c = croniter("0 10 * * * ", start_time=dt)
-        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
-        self.assertEqual(ts, '2018-01-02 10:00:00')
-        #
+        self.assertEqual(
+            _croniter(start_time=dt).get_prev(),
+            datetime(2018, 1, 2, 10, 0),
+        )
+        self.assertEqual(
+            _croniter(start_time=dt).get_next(),
+            datetime(2018, 1, 3, 10, 0),
+        )
+
         dt = datetime(2018, 1, 2, 9, 59, 59, 999999)
-        c = croniter("0 10 * * * ", start_time=dt)
-        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
-        self.assertEqual(ts, '2018-01-01 10:00:00')
+        self.assertEqual(
+            _croniter(start_time=dt).get_prev(),
+            datetime(2018, 1, 1, 10, 0),
+        )
+        self.assertEqual(
+            _croniter(start_time=dt).get_next(),
+            datetime(2018, 1, 2, 10, 0),
+        )
 
     def test_invalid_zerorepeat(self):
         self.assertFalse(croniter.is_valid('*/0 * * * *'))
@@ -845,7 +930,7 @@ class CroniterTest(base.TestCase):
             ret.append(dt)
             dt += timedelta(days=1)
         sret = ["{0}".format(r) for r in ret]
-        self.assertEquals(
+        self.assertEqual(
             sret,
             ['2019-01-15 00:00:00',
              '2019-01-16 00:00:01',
@@ -865,7 +950,7 @@ class CroniterTest(base.TestCase):
             ret.append(dt)
             dt += timedelta(days=1)
         sret = ["{0}".format(r) for r in ret]
-        self.assertEquals(
+        self.assertEqual(
             sret,
             ['2019-01-14 00:00:01',
              '2019-01-15 00:00:02',
@@ -888,7 +973,7 @@ class CroniterTest(base.TestCase):
             ret.append(dt)
             dt += timedelta(days=1)
         sret = ["{0}".format(r) for r in ret]
-        self.assertEquals(
+        self.assertEqual(
             sret,
             ['2019-01-16 00:00:00',
              '2019-01-17 00:00:01',
@@ -901,6 +986,202 @@ class CroniterTest(base.TestCase):
              '2019-01-26 00:00:03',
              '2019-01-27 00:00:04'])
 
+    def test_mixdow(self):
+        base = datetime(2018, 10, 1, 0, 0)
+        itr = croniter('1 1 7,14,21,L * *', base)
+        self.assertTrue(isinstance(itr.get_next(), float))
+
+    def test_match(self):
+        self.assertTrue(croniter.match(
+            "0 0 * * *",
+            datetime(2019, 1, 14, 0, 0, 0, 0)
+        ))
+        self.assertFalse(croniter.match(
+            "0 0 * * *",
+            datetime(2019, 1, 14, 0, 1, 0, 0)
+        ))
+        self.assertTrue(croniter.match(
+            "31 * * * *",
+            datetime(2019, 1, 14, 1, 31, 0, 0)
+        ))
+        self.assertTrue(croniter.match(
+            "0 0 10 * wed",
+            datetime(2020, 6, 10, 0, 0, 0, 0),
+            day_or=True
+        ))
+        self.assertTrue(croniter.match(
+            "0 0 10 * fri",
+            datetime(2020, 6, 10, 0, 0, 0, 0),
+            day_or=True
+        ))
+        self.assertTrue(croniter.match(
+            "0 0 10 * fri",
+            datetime(2020, 6, 12, 0, 0, 0, 0),
+            day_or=True
+        ))
+        self.assertTrue(croniter.match(
+            "0 0 10 * wed",
+            datetime(2020, 6, 10, 0, 0, 0, 0),
+            day_or=False
+        ))
+        self.assertFalse(croniter.match(
+            "0 0 10 * fri",
+            datetime(2020, 6, 10, 0, 0, 0, 0),
+            day_or=False
+        ))
+        self.assertFalse(croniter.match(
+            "0 0 10 * fri",
+            datetime(2020, 6, 12, 0, 0, 0, 0),
+            day_or=False
+        ))
+
+
+    def test_dst_issue90_st31ny(self):
+        tz = pytz.timezone("Europe/Paris")
+        now = datetime(2020, 3, 29, 1, 59, 55, tzinfo=tz)
+        it = croniter('1 2 * * *', now)
+        #
+        # Taking around DST @ 29/03/20 01:59
+        #
+        ret = [
+            it.get_next(datetime).isoformat(),
+            it.get_prev(datetime).isoformat(),
+            it.get_prev(datetime).isoformat(),
+            it.get_next(datetime).isoformat(),
+            it.get_next(datetime).isoformat(),
+        ]
+        self.assertEqual(ret, [
+            '2020-03-30T02:01:00+02:00',
+            '2020-03-29T01:01:00+01:00',
+            '2020-03-28T03:01:00+01:00',
+            '2020-03-29T02:01:00+02:00',
+            '2020-03-29T03:01:00+02:00'])
+        #
+        nowp = datetime(2020, 3, 28, 1, 58, 55, tzinfo=tz)
+        itp = croniter('1 2 * * *', nowp)
+        retp = [
+            itp.get_next(datetime).isoformat(),
+            itp.get_prev(datetime).isoformat(),
+            itp.get_prev(datetime).isoformat(),
+            itp.get_next(datetime).isoformat(),
+            itp.get_next(datetime).isoformat(),
+        ]
+        self.assertEqual(retp, [
+            '2020-03-29T02:01:00+02:00',
+            '2020-03-28T02:01:00+01:00',
+            '2020-03-27T02:01:00+01:00',
+            '2020-03-28T02:01:00+01:00',
+            '2020-03-29T03:01:00+02:00'])
+        #
+        nowt = datetime(2020, 3, 29, 2, 0, 0, tzinfo=tz)
+        itt = croniter('1 2 * * *', nowt)
+        rett = [
+            itt.get_next(datetime).isoformat(),
+            itt.get_prev(datetime).isoformat(),
+            itt.get_prev(datetime).isoformat(),
+            itt.get_next(datetime).isoformat(),
+            itt.get_next(datetime).isoformat(),
+        ]
+        self.assertEqual(rett, [
+            '2020-03-30T02:01:00+02:00',
+            '2020-03-29T01:01:00+01:00',
+            '2020-03-28T03:01:00+01:00',
+            '2020-03-29T02:01:00+02:00',
+            '2020-03-29T03:01:00+02:00'])
+
+
+class CroniterRangeTest(base.TestCase):
+
+    def test_1day_step(self):
+        start = datetime(2016, 12, 2)
+        stop = datetime(2016, 12, 10)
+        fwd = list(croniter_range(start, stop, '0 0 * * *'))
+        self.assertEqual(len(fwd), 9)
+        self.assertEqual(fwd[0], start)
+        self.assertEqual(fwd[-1], stop)
+        # Test the same, but in reverse
+        rev = list(croniter_range(stop, start, '0 0 * * *'))
+        self.assertEqual(len(rev), 9)
+        # Ensure forward/reverse are a mirror image
+        rev.reverse()
+        self.assertEqual(fwd, rev)
+
+    def test_1day_step_no_ends(self):
+        # Test without ends (exclusive)
+        start = datetime(2016, 12, 2)
+        stop = datetime(2016, 12, 10)
+        fwd = list(croniter_range(start, stop, '0 0 * * *', exclude_ends=True))
+        self.assertEqual(len(fwd), 7)
+        self.assertNotEqual(fwd[0], start)
+        self.assertNotEqual(fwd[-1], stop)
+        # Test the same, but in reverse
+        rev = list(croniter_range(stop, start, '0 0 * * *', exclude_ends=True))
+        self.assertEqual(len(rev), 7)
+        self.assertNotEqual(fwd[0], stop)
+        self.assertNotEqual(fwd[-1], start)
+
+    def test_1month_step(self):
+        start = datetime(1982, 1, 1)
+        stop = datetime(1983, 12, 31)
+        res = list(croniter_range(start, stop, '0 0 1 * *'))
+        self.assertEqual(len(res), 24)
+        self.assertEqual(res[0], start)
+        self.assertEqual(res[5].day, 1)
+        self.assertEqual(res[-1], datetime(1983, 12, 1))
+
+    def test_1minute_step_float(self):
+        start = datetime(2000, 1, 1, 0, 0)
+        stop =  datetime(2000, 1, 1, 0, 1)
+        res = list(croniter_range(start, stop, '* * * * *', ret_type=float))
+        self.assertEqual(len(res), 2)
+        self.assertEqual(res[0], 946684800.0)
+        self.assertEqual(res[-1] - res[0], 60)
+
+    def test_auto_ret_type(self):
+        data = [
+            (datetime(2019, 1, 1), datetime(2020, 1, 1), datetime),
+            (1552252218.0, 1591823311.0, float),
+        ]
+        for start, stop, rtype in data:
+            ret = list(croniter_range(start, stop, "0 0 * * *"))
+            self.assertIsInstance(ret[0], rtype)
+
+    def test_input_type_exceptions(self):
+        dt_start1 = datetime(2019, 1, 1)
+        dt_stop1 = datetime(2020, 1, 1)
+        f_start1 = 1552252218.0
+        f_stop1 = 1591823311.0
+        # Mix start/stop types
+        with self.assertRaises(TypeError):
+            list(croniter_range(dt_start1, f_stop1, "0 * * * *"), ret_type=datetime)
+        with self.assertRaises(TypeError):
+            list(croniter_range(f_start1, dt_stop1, "0 * * * *"))
+
+    def test_timezone_dst(self):
+        """ Test across DST transition, which technially is a timzone change. """
+        tz = pytz.timezone("US/Eastern")
+        start = tz.localize(datetime(2020, 10, 30))
+        stop =  tz.localize(datetime(2020, 11, 10))
+        res = list(croniter_range(start, stop, '0 0 * * *'))
+        self.assertNotEqual(res[0].tzinfo, res[-1].tzinfo)
+        self.assertEqual(len(res), 12)
+
+    def test_extra_hour_day_prio(self):
+        def datetime_tz(*args, **kw):
+            """ Defined this in another branch.  single-use-version """
+            tzinfo = kw.pop("tzinfo")
+            return tzinfo.localize(datetime(*args))
+        tz = pytz.timezone("US/Eastern")
+        cron = "0 3 * * *"
+        start = datetime_tz(2020, 3, 7, tzinfo=tz)
+        end = datetime_tz(2020, 3, 11, tzinfo=tz)
+        ret = [ i.isoformat() for i in croniter_range(start, end, cron) ]
+        self.assertEqual(ret, [
+            "2020-03-07T03:00:00-05:00",
+            "2020-03-08T03:00:00-04:00",
+            "2020-03-09T03:00:00-04:00",
+            "2020-03-10T03:00:00-04:00"])
+
 
 if __name__ == '__main__':
     unittest.main()
